@@ -1,18 +1,32 @@
+from __future__ import division
 from django.db import models
-from django.utils.encoding import smart_unicode
-from exceptions import Exception
+from django.conf import settings
+try:
+    from django.utils.encoding import smart_unicode
+except ImportError:
+    # Python 3
+    from django.utils.encoding import smart_text as smart_unicode
+from django.utils import translation
+from django.db.models.signals import class_prepared
 from moneyed import Money, Currency, DEFAULT_CURRENCY
+from moneyed.localization import _FORMATTER, format_money
 from djmoney import forms
 from djmoney.forms.widgets import CURRENCY_CHOICES
+from djmoney.utils import get_currency_field_name
 from django.db.models.expressions import ExpressionNode
 
-from decimal import Decimal
+from decimal import Decimal, ROUND_DOWN
 import inspect
 
-__all__ = ('MoneyField', 'currency_field_name', 'NotSupportedLookup')
+try:
+    unicode = unicode
+except NameError:
+    # 'unicode' is undefined, in Python 3
+    basestring = (str, bytes)
 
-currency_field_name = lambda name: "%s_currency" % name
-SUPPORTED_LOOKUPS = ('exact', 'lt', 'gt', 'lte', 'gte')
+__all__ = ('MoneyField', 'NotSupportedLookup')
+
+SUPPORTED_LOOKUPS = ('exact', 'isnull', 'lt', 'gt', 'lte', 'gte')
 
 
 class NotSupportedLookup(Exception):
@@ -24,21 +38,120 @@ class NotSupportedLookup(Exception):
 
 
 class MoneyPatched(Money):
+
+    # Set to True or False has a higher priority
+    # than USE_L10N == True in the django settings file.
+    # The variable "self.use_l10n" has three states:
+    use_l10n = None
+
     def __float__(self):
         return float(self.amount)
 
+    @classmethod
+    def _patch_to_current_class(cls, money):
+        """
+        Converts object of type MoneyPatched on the object of type Money.
+        """
+        return cls(money.amount, money.currency)
+
+    def __pos__(self):
+        return MoneyPatched._patch_to_current_class(
+            super(MoneyPatched, self).__pos__())
+
+    def __neg__(self):
+        return MoneyPatched._patch_to_current_class(
+            super(MoneyPatched, self).__neg__())
+
+    def __add__(self, other):
+
+        return MoneyPatched._patch_to_current_class(
+            super(MoneyPatched, self).__add__(other))
+
+    def __sub__(self, other):
+
+        return MoneyPatched._patch_to_current_class(
+            super(MoneyPatched, self).__sub__(other))
+
+    def __mul__(self, other):
+
+        return MoneyPatched._patch_to_current_class(
+            super(MoneyPatched, self).__mul__(other))
+
+    def __truediv__(self, other):
+
+        if isinstance(other, Money):
+            return super(MoneyPatched, self).__truediv__(other)
+        else:
+            return self._patch_to_current_class(
+                super(MoneyPatched, self).__truediv__(other))
+
+    def __rmod__(self, other):
+
+        return MoneyPatched._patch_to_current_class(
+            super(MoneyPatched, self).__rmod__(other))
+
+    def __get_current_locale(self):
+        locale = translation.get_language()
+
+        if _FORMATTER.get_formatting_definition(locale):
+            return locale
+
+        if _FORMATTER.get_formatting_definition('%s_%s' % (locale, locale)):
+            return '%s_%s' % (locale, locale)
+
+        return ''
+
+    def __use_l10n(self):
+        'Return boolean'
+
+        # Do not change. The variable "self.use_l10n" has three states:
+        # True, False, and None.
+        if self.use_l10n == True:
+            return True
+
+        # Do not change. The variable "self.use_l10n" has three states:
+        # True, False, and None.
+        if self.use_l10n == False:
+            return False
+
+        # if self.use_l10n == None >>
+        return settings.USE_L10N
+
+    def __unicode__(self):
+
+        if self.__use_l10n():
+            locale = self.__get_current_locale()
+            if locale:
+                return format_money(self, locale=locale)
+
+        return format_money(self)
+
+    def __str__(self):
+
+        if self.__use_l10n():
+            locale = self.__get_current_locale()
+            if locale:
+                return format_money(self, locale=locale)
+
+        return format_money(self)
+
+    def __repr__(self):
+        # small fix for tests
+        return "%s %s" % (self.amount.to_integral_value(ROUND_DOWN),
+                          self.currency)
+
 
 class MoneyFieldProxy(object):
     def __init__(self, field):
         self.field = field
-        self.currency_field_name = currency_field_name(self.field.name)
+        self.currency_field_name = get_currency_field_name(self.field.name)
 
     def _money_from_obj(self, obj):
-        value = obj.__dict__[self.field.name], obj.__dict__[
-            self.currency_field_name]
-        if value[0] is None:
+        amount = obj.__dict__[self.field.name]
+        currency = obj.__dict__[self.currency_field_name]
+        if amount is None:
             return None
-        return MoneyPatched(amount=value[0], currency=value[1])
+        return MoneyPatched(amount=amount, currency=currency)
 
     def __get__(self, obj, type=None):
         if obj is None:
@@ -86,6 +199,7 @@ class CurrencyField(models.CharField):
         if not self.frozen_by_south and not name in [f.name for f in cls._meta.fields]:
             super(CurrencyField, self).contribute_to_class(cls, name)
 
+
 class MoneyField(models.DecimalField):
     description = "A field which stores both the currency and amount of money."
 
@@ -95,13 +209,10 @@ class MoneyField(models.DecimalField):
                  default_currency=DEFAULT_CURRENCY,
                  currency_choices=CURRENCY_CHOICES, **kwargs):
 
-
         if isinstance(default, basestring):
             amount, currency = default.split(" ")
             default = Money(float(amount), Currency(code=currency))
-        elif isinstance(default, float):
-            default = Money(default, default_currency)
-        elif isinstance(default, Decimal):
+        elif isinstance(default, (float, Decimal)):
             default = Money(default, default_currency)
 
         if not isinstance(default, Money):
@@ -118,7 +229,6 @@ class MoneyField(models.DecimalField):
             raise Exception(
                 "You have to provide a decimal_places attribute to Money fields.")
 
-
         if not default_currency:
             default_currency = default.currency
 
@@ -142,12 +252,15 @@ class MoneyField(models.DecimalField):
 
     def contribute_to_class(self, cls, name):
 
+        cls._meta.has_money_field = True
+
         # Don't run on abstract classes
-        if cls._meta.abstract:
-            return
+        # Removed, see https://github.com/jakewins/django-money/issues/42
+        #if cls._meta.abstract:
+        #    return
 
         if not self.frozen_by_south:
-            c_field_name = currency_field_name(name)
+            c_field_name = get_currency_field_name(name)
             # Do not change default=self.default_currency.code, needed
             # for south compat.
             c_field = CurrencyField(
@@ -162,20 +275,6 @@ class MoneyField(models.DecimalField):
 
         setattr(cls, self.name, MoneyFieldProxy(self))
 
-        from managers import money_manager
-
-        if getattr(cls, '_default_manager', None):
-            cls._default_manager = money_manager(cls._default_manager)
-        else:
-            cls._default_manager = money_manager(models.Manager())
-        cls._default_manager.model = cls
-
-        if getattr(cls, 'objects', None):
-            cls.objects = money_manager(cls.objects)
-        else:
-            cls.objects = money_manager(models.Manager())
-        cls.objects.model = cls
-
     def get_db_prep_save(self, value, connection):
         if isinstance(value, Money):
             value = value.amount
@@ -217,7 +316,6 @@ class MoneyField(models.DecimalField):
         value = self._get_val_from_obj(obj)
         return self.get_prep_value(value)
 
-
     ## South support
     def south_field_triple(self):
         "Returns a suitable description of this field for South."
@@ -230,7 +328,7 @@ class MoneyField(models.DecimalField):
         kwargs.pop('default')
         # 2. add the default currency, because it's not picked up from the inspector automatically.
         kwargs['default_currency'] = "'%s'" % self.default_currency
-        return (field_class, args, kwargs)
+        return field_class, args, kwargs
 
 
 try:
@@ -247,3 +345,17 @@ try:
     add_introspection_rules(rules, ["^djmoney\.models\.fields\.CurrencyField"])
 except ImportError:
     pass
+
+
+def patch_managers(sender, **kwargs):
+    """
+    Patches models managers
+    """
+    from .managers import money_manager
+
+    if hasattr(sender._meta, 'has_money_field'):
+        for _id, name, manager in sender._meta.concrete_managers:
+            setattr(sender, name, money_manager(manager))
+
+
+class_prepared.connect(patch_managers)
